"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getInterpolationRefs = getInterpolationRefs;
exports.getInterpolatedTemplate = getInterpolatedTemplate;
exports.INTERPOLATION_REFERENCE_REGEX = exports.INTERPOLATION_REFERENCE_ID = void 0;

/**
 * The interpolation reference is used to indicate
 * that an interpolation needs to occur when encountered
 * by a visitor.
 */
const INTERPOLATION_REFERENCE_ID = '_react_pug_replace_';
/**
 * Used to check whether a value contains a replace
 * reference or multiple replace references.
 */

exports.INTERPOLATION_REFERENCE_ID = INTERPOLATION_REFERENCE_ID;
const INTERPOLATION_REFERENCE_REGEX = /_react_pug_replace_\d+/g;
/**
 * Check whether the value is a valid interpolation
 * reference.
 * @param { string } value - The value to check
 * @returns { ?Array<string> } The references within
 * the value or null.
 */

exports.INTERPOLATION_REFERENCE_REGEX = INTERPOLATION_REFERENCE_REGEX;

function getInterpolationRefs(value) {
  return value.match(INTERPOLATION_REFERENCE_REGEX);
}
/**
 * Convert pug raw template array into a string
 * containing references to interpolations as well
 * a map containing the interpolations.
 * @param { Array<TemplateElement> } tpl - The template array
 * @param { Array<Expression> } interpolations - The interpolations
 * @returns { Object } - The template with interpolation references
 * and a map containing the reference and the interpolation.
 */


function getInterpolatedTemplate(tpl, interpolations) {
  const interpolationRef = new Map();
  const template = tpl.map(({
    value
  }, index) => {
    const interpolation = interpolations[index];
    const rawValue = value && typeof value === 'object' ? value.raw : '';

    if (interpolation) {
      const ref = `${INTERPOLATION_REFERENCE_ID}${index}`;
      interpolationRef.set(ref, interpolation);
      return `${String(rawValue)}${ref}`;
    }

    return rawValue;
  }).join('');
  return {
    template,
    interpolationRef
  };
}